@@ -1,7 +1,7 @@
 /* pdc640.c
  *
- * Copyright  2001 Lutz Mller <lutz@users.sourceforge.net>
- * Copyright  2002 Marcus Meissner <marcus@jet.franken.de>
+ * Copyright 2001 Lutz Mueller <lutz@users.sourceforge.net>
+ * Copyright 2002 Marcus Meissner <marcus@jet.franken.de>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 #include "config.h"
 
@@ -24,11 +24,12 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <gphoto2-library.h>
-#include <gphoto2-port-log.h>
+#include <gphoto2/gphoto2-library.h>
+#include <gphoto2/gphoto2-port-log.h>
 #include <bayer.h>
 
 #include "jd350e.h"
+#include "dlink350f.h"
 
 #define GP_MODULE "pdc640"
 
@@ -37,7 +38,7 @@
 
 #define PDC640_MAXTRIES 3
 
-#define CHECK_RESULT(result) {int r = (result); if (r < 0) return (r);}
+#define CHECK_RESULT(result) {int __r = (result); if (__r < 0) return (__r);}
 
 #ifdef ENABLE_NLS
 #  include <libintl.h>
@@ -61,7 +62,8 @@
 
 typedef enum{
 	pdc640,
-	jd350e
+	jd350e,
+	dlink350f
 } Model;
 
 typedef int postproc_func(int,int,unsigned char*);
@@ -105,32 +107,32 @@ static struct {
 	{"Jenoptik JD350 video", 0xd96, 0x0000, {
 		 	jd350e,
 			BAYER_TILE_BGGR,
-			&jd350e_postprocessing,
+			NULL /*&jd350e_postprocessing*/,
 			"jd350v%04i.ppm"
 		}
 	},
-	{"ScanHex SX-35", 0x797, 0x8901, {
+	{"ScanHex SX-35a", 0x797, 0x8901, {
 		 	jd350e,
 			BAYER_TILE_BGGR,
 			&jd350e_postprocessing,
 			"jd350v%04i.ppm"
 		}
 	},
-	{"ScanHex SX-35", 0x797, 0x8909, {
+	{"ScanHex SX-35b", 0x797, 0x8909, {
 		 	jd350e,
 			BAYER_TILE_BGGR,
 			&jd350e_postprocessing,
 			"jd350v%04i.ppm"
 		}
 	},
-	{"ScanHex SX-35", 0x797, 0x8911, {
+	{"ScanHex SX-35c", 0x797, 0x8911, {
 		 	jd350e,
 			BAYER_TILE_BGGR,
 			&jd350e_postprocessing,
 			"jd350v%04i.ppm"
 		}
 	},
-	{"ScanHex SX-35", 0x84d, 0x1001, {
+	{"ScanHex SX-35d", 0x84d, 0x1001, {
 		 	jd350e,
 			BAYER_TILE_BGGR,
 			&jd350e_postprocessing,
@@ -151,6 +153,20 @@ static struct {
 			"trust%04i.ppm"
 		}
 	},
+	{"Trust PowerC@m 350FT", 0x6d6, 0x002d, {
+			jd350e,
+			BAYER_TILE_RGGB,
+			&trust350fs_postprocessing,
+			"trust%04i.ppm"
+		}
+	},
+	{"Clever CAM 360", 0x797, 0x8001, {
+			jd350e,
+			BAYER_TILE_BGGR,
+			&jd350e_postprocessing_and_flip,
+			"scope%04i.ppm"
+		}
+	},
 	/* http://www.meade.com/sportsoptics/catalog/captureview/index.html */
 	{"GrandTek ScopeCam", 0x797, 0x801c, {
 		 	jd350e,
@@ -182,9 +198,9 @@ static struct {
 	/* http://www.dlink.com/products/usb/dsc350/ */
 	/* ids from driver download */
 	{"D-Link DSC 350F", 0xd64, 0x1021, {
-		 	jd350e,
+		 	dlink350f,
 			BAYER_TILE_BGGR,
-			&flip_vertical,
+			&dlink_dsc350f_postprocessing_and_flip_both,
 			"dlink%04i.ppm"
 		}
 	},
@@ -192,7 +208,7 @@ static struct {
 };
 
 static int
-pdc640_read_packet (GPPort *port, char *buf, int buf_size)
+pdc640_read_packet (GPPort *port, unsigned char *buf, int buf_size)
 {
 	int i;
 	char checksum, c;
@@ -202,7 +218,7 @@ pdc640_read_packet (GPPort *port, char *buf, int buf_size)
 		buf[i] = 0;
 
 	/* Read the packet */
-	CHECK_RESULT (gp_port_read (port, buf, buf_size));
+	CHECK_RESULT (gp_port_read (port, (char*)buf, buf_size));
 
 	/* Calculate the checksum */
 	for (i = 0, checksum = 0; i < buf_size; i++)
@@ -218,8 +234,8 @@ pdc640_read_packet (GPPort *port, char *buf, int buf_size)
 }
 
 static int
-pdc640_transmit (GPPort *port, char *cmd, int cmd_size,
-				   char *buf, int buf_size)
+pdc640_transmit (GPPort *port, unsigned char *cmd, int cmd_size,
+				   unsigned char *buf, int buf_size)
 {
 	int r, tries;
 
@@ -234,7 +250,7 @@ pdc640_transmit (GPPort *port, char *cmd, int cmd_size,
 		xcmd[3] = checksum & 0xff;
 
 		/* Wait until we get back the echo of the command. */
-		r = gp_port_usb_msg_read( port, 0x10, xcmd[0]|(xcmd[1]<<8),xcmd[2]|(xcmd[3]<<8),xbuf,sizeof(xbuf));
+		r = gp_port_usb_msg_read( port, 0x10, xcmd[0]|(xcmd[1]<<8),xcmd[2]|(xcmd[3]<<8),(char*)xbuf,sizeof(xbuf));
 			/* Sometimes we want to read here, sometimes not.
 		if (r < GP_OK)
 			return r;
@@ -245,7 +261,7 @@ pdc640_transmit (GPPort *port, char *cmd, int cmd_size,
 			 */
 			int curr = 0, readsize = (buf_size + 63) & ~63;
 			while (curr < readsize) {
-				r = gp_port_read( port, buf + curr, readsize - curr);
+				r = gp_port_read( port, (char*)buf + curr, readsize - curr);
 				if (r < GP_OK) break;
 				curr += r;
 			}
@@ -260,7 +276,7 @@ pdc640_transmit (GPPort *port, char *cmd, int cmd_size,
 			 * The first byte returned is always the same as the first byte
 			 * of the command.
 			 */
-			CHECK_RESULT (gp_port_write (port, cmd, cmd_size));
+			CHECK_RESULT (gp_port_write (port, (char*)cmd, cmd_size));
 			r = gp_port_read (port, &c, 1);
 			if ((r < 0) || (c != cmd[0]))
 				continue;
@@ -280,10 +296,10 @@ pdc640_transmit (GPPort *port, char *cmd, int cmd_size,
 
 static int
 pdc640_transmit_pic (GPPort *port, char cmd, int width, int thumbnail,
-		     char *buf, int buf_size)
+		     unsigned char *buf, int buf_size)
 {
-	char cmd1[] = {0x61, 0x00};
-	char cmd2[] = {0x15, 0x00, 0x00, 0x00, 0x00};
+	unsigned char cmd1[] = {0x61, 0x00};
+	unsigned char cmd2[] = {0x15, 0x00, 0x00, 0x00, 0x00};
 
 	/* First send the command ... */
 	cmd1[1] = cmd;
@@ -295,7 +311,7 @@ pdc640_transmit_pic (GPPort *port, char cmd, int width, int thumbnail,
 		return pdc640_transmit (port, cmd2, 4, buf, buf_size);
 	} else {
 		int i, packet_size, result, size, ofs;
-		char *data;
+		unsigned char *data;
 
 		/* Set how many scanlines worth of data to get at once */
 		cmd2[4] = 0x06;
@@ -335,20 +351,20 @@ pdc640_transmit_pic (GPPort *port, char cmd, int width, int thumbnail,
 }
 
 static int
-pdc640_transmit_packet (GPPort *port, char cmd, char *buf, int buf_size) {
-	char cmd1[] = {0x61, 0x00};
+pdc640_transmit_packet (GPPort *port, char cmd, unsigned char *buf, int buf_size) {
+	unsigned char cmd1[] = {0x61, 0x00};
 	/* Send the command and get the packet */
 	cmd1[1] = cmd;
 	CHECK_RESULT (pdc640_transmit (port, cmd1, 2, NULL, 0));
 
 	if (port->type == GP_PORT_USB) {
-		char cmd2[] = {0x15, 0x00, 0x00, 0x00};
+		unsigned char cmd2[] = {0x15, 0x00, 0x00, 0x00};
 
 		cmd2[1] = ((buf_size+63)/64) & 0xff;
 		cmd2[2] = (((buf_size+63)/64) >> 8) & 0xff;
 		return pdc640_transmit (port, cmd2, 4, buf, buf_size);
 	} else {
-		char cmd2[] = {0x15, 0x00, 0x00, 0x00, 0x01};
+		unsigned char cmd2[] = {0x15, 0x00, 0x00, 0x00, 0x01};
 
 		return pdc640_transmit (port, cmd2, 5, buf, buf_size);
 	}
@@ -358,7 +374,7 @@ pdc640_transmit_packet (GPPort *port, char cmd, char *buf, int buf_size) {
 static int
 pdc640_ping_low (GPPort *port)
 {
-	char cmd[] = {0x01};
+	unsigned char cmd[] = {0x01};
 
 	CHECK_RESULT (pdc640_transmit (port, cmd, 1, NULL, 0));
 
@@ -380,7 +396,7 @@ pdc640_push_button (GPPort *port)
 static int
 pdc640_ping_high (GPPort *port)
 {
-	char cmd[] = {0x41};
+	unsigned char cmd[] = {0x41};
 
 	CHECK_RESULT (pdc640_transmit (port, cmd, 1, NULL, 0));
 
@@ -390,7 +406,7 @@ pdc640_ping_high (GPPort *port)
 static int
 pdc640_speed (GPPort *port, int speed)
 {
-	char cmd[] = {0x69, 0x00};
+	unsigned char cmd[] = {0x69, 0x00};
 
 	cmd[1] = (speed / 9600) - 1;
 	CHECK_RESULT (pdc640_transmit (port, cmd, 2, NULL, 0));
@@ -419,7 +435,7 @@ pdc640_unknown5 (GPPort *port)
 static int
 pdc640_unknown20 (GPPort* port)
 {
-	char buf[128];
+	unsigned char buf[128];
 
 	CHECK_RESULT (pdc640_transmit_packet (port, 0x20, buf, 128));
 
@@ -430,7 +446,7 @@ pdc640_unknown20 (GPPort* port)
 static int
 pdc640_caminfo (GPPort *port, int *numpic)
 {
-	char buf[1280];
+	unsigned char buf[1280];
 
 	CHECK_RESULT (pdc640_transmit_packet (port, 0x40, buf, 1280));
 	*numpic = buf[2]; /* thats the only useful info :( */
@@ -440,14 +456,14 @@ pdc640_caminfo (GPPort *port, int *numpic)
 static int
 pdc640_setpic (GPPort *port, char n)
 {
-	char cmd[2] = {0xf6, 0x00};
+	unsigned char cmd[2] = {0xf6, 0x00};
 
 	cmd[1] = n;
 	if (port->type == GP_PORT_USB) {
 		/* USB does not like a bulkread afterwards */
 		return pdc640_transmit (port, cmd, 2, NULL, 0);
 	} else {
-		char buf[8];
+		unsigned char buf[8];
 
 		return pdc640_transmit (port, cmd, 2, buf, 7);
 	}
@@ -490,8 +506,8 @@ pdc640_picinfo (GPPort *port, char n,
 }
 
 static int
-pdc640_processtn (int width, int height, char **data, int size) {
-	char *newdata;
+pdc640_processtn (int width, int height, unsigned char **data, int size) {
+	unsigned char *newdata;
 	int y;
 
 	/* Sanity checks */
@@ -517,7 +533,7 @@ pdc640_processtn (int width, int height, char **data, int size) {
 }
 
 static int
-pdc640_getbit (char *data, int *ofs, int size, int *bit) {
+pdc640_getbit (unsigned char *data, int *ofs, int size, int *bit) {
 	static char c;
 	int b;
 
@@ -542,10 +558,10 @@ pdc640_getbit (char *data, int *ofs, int size, int *bit) {
 }
 
 static int
-pdc640_deltadecode (int width, int height, char **rawdata, int *rawsize)
+pdc640_deltadecode (int width, int height, unsigned char **rawdata, int *rawsize)
 {
 	char col1, col2;
-	char *data;
+	unsigned char *data;
 	int rawofs, x, y, ofs, bit, ones;
 	int size;
 	int e, d, o, val;
@@ -620,14 +636,14 @@ pdc640_deltadecode (int width, int height, char **rawdata, int *rawsize)
 
 static int
 pdc640_getpic (Camera *camera, int n, int thumbnail, int justraw,
-		char **data, int *size)
+		unsigned char **data, int *size)
 {
 	char cmd, ppmheader[100];
 	int size_pic, width_pic, height_pic;
 	int size_thumb, width_thumb, height_thumb;
 	int height, width, outsize, result, pmmhdr_len;
 	int compression_type;
-	char *outdata;
+	unsigned char *outdata;
 
 	/* Get the size of the picture */
 	CHECK_RESULT (pdc640_picinfo (camera->port, n,
@@ -709,7 +725,7 @@ pdc640_getpic (Camera *camera, int n, int thumbnail, int justraw,
 		return (GP_ERROR_NO_MEMORY);
 
 	/* Set header */
-	strcpy(outdata, ppmheader);
+	strcpy((char *)outdata, ppmheader);
 
 	/* Decode and interpolate the Bayer Mask */
 	result = gp_bayer_decode(*data, width, height,
@@ -739,7 +755,7 @@ pdc640_getpic (Camera *camera, int n, int thumbnail, int justraw,
 static int
 pdc640_delpic (GPPort *port)
 {
-	char cmd[2] = {0x59, 0x01};
+	unsigned char cmd[2] = {0x59, 0x01};
 
 	CHECK_RESULT (pdc640_transmit (port, cmd, 2, NULL, 0));
 
@@ -749,7 +765,7 @@ pdc640_delpic (GPPort *port)
 static int
 pdc640_takepic (GPPort *port)
 {
-	char cmd[2] = {0x2D, 0x00};
+	unsigned char cmd[2] = {0x2D, 0x00};
 
 	CHECK_RESULT (pdc640_transmit (port, cmd, 2, NULL, 0));
 
@@ -801,7 +817,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 {
 	Camera *camera = user_data;
 	int n, size;
-	char *data, *p;
+	unsigned char *data;
 
 	/*
 	 * Get the number of the picture from the filesystem and increment
@@ -811,8 +827,6 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 						context));
 	n++;
 
-	CHECK_RESULT (gp_file_set_name (file, filename));
-
 	/* Get the picture */
 	switch (type) {
 	case GP_FILE_TYPE_NORMAL:
@@ -822,18 +836,6 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	case GP_FILE_TYPE_RAW:
 		CHECK_RESULT (pdc640_getpic (camera, n, 0, 1, &data, &size));
 		CHECK_RESULT (gp_file_set_mime_type (file, GP_MIME_RAW));
-
-		/* Change extension to raw */
-		n = strlen(filename);
-		p = malloc (n + 1);
-		if (p) {
-			strcpy (p, filename);
-			p[n-3] = 'r';
-			p[n-2] = 'a';
-			p[n-1] = 'w';
-			CHECK_RESULT (gp_file_set_name (file, p));
-			free (p);
-		}
 		break;
 	case GP_FILE_TYPE_PREVIEW:
 		CHECK_RESULT (pdc640_getpic (camera, n, 1, 0, &data, &size));
@@ -843,7 +845,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		return (GP_ERROR_NOT_SUPPORTED);
 	}
 
-	CHECK_RESULT (gp_file_set_data_and_size (file, data, size));
+	CHECK_RESULT (gp_file_set_data_and_size (file, (char*)data, size));
 
 	return (GP_OK);
 }
@@ -853,7 +855,7 @@ delete_all_func (CameraFilesystem *fs, const char *folder, void *data,
 		 GPContext *context)
 {
 	Camera *camera = data;
-	char cmd[2] = {0x59, 0x00};
+	unsigned char cmd[2] = {0x59, 0x00};
 
 	CHECK_RESULT (pdc640_transmit (camera->port, cmd, 2, NULL, 0));
 
@@ -889,8 +891,8 @@ camera_about (Camera *camera, CameraText *about, GPContext *context)
 		"and adapted for gphoto2 by Lutz Mueller "
 		"<lutz@users.sf.net>."
 		"Protocol enhancements and postprocessing "
-		" for Jenoptik JD350e by Michael Trawny "
-		"<trawny99@users.sourceforge.net>."
+		"for Jenoptik JD350e by Michael Trawny "
+		"<trawny99@users.sourceforge.net>. "
 		"Bugfixes by Marcus Meissner <marcus@jet.franken.de>."));
 
 	return (GP_OK);
@@ -991,6 +993,14 @@ camera_exit (Camera *camera, GPContext *context)
 	return (GP_OK);
 }
 
+static CameraFilesystemFuncs fsfuncs = {
+	.file_list_func = file_list_func,
+	.get_info_func = get_info_func,
+	.get_file_func = get_file_func,
+	.del_file_func = delete_file_func,
+	.delete_all_func = delete_all_func,
+};
+
 int
 camera_init (Camera *camera, GPContext *context)
 {
@@ -1025,16 +1035,8 @@ camera_init (Camera *camera, GPContext *context)
 	if( ! camera->pl ){
 		return (GP_ERROR_NOT_SUPPORTED);
 	}
-
 	/* Tell the filesystem where to get lists and info */
-	CHECK_RESULT (gp_filesystem_set_list_funcs (camera->fs, file_list_func,
-						    NULL, camera));
-	CHECK_RESULT (gp_filesystem_set_info_funcs (camera->fs, get_info_func,
-						    NULL, camera));
-	CHECK_RESULT (gp_filesystem_set_folder_funcs (camera->fs, NULL,
-					delete_all_func, NULL, NULL, camera));
-	CHECK_RESULT (gp_filesystem_set_file_funcs (camera->fs, get_file_func,
-						delete_file_func, camera));
+	CHECK_RESULT (gp_filesystem_set_funcs (camera->fs, &fsfuncs, camera));
 
 	if (camera->port->type == GP_PORT_SERIAL) {
 		/* Open the port */
