@@ -23,7 +23,6 @@
 #include <gphoto2/gphoto2-port-library.h>
 
 #include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #ifdef HAVE_FCNTL_H
 # include <fcntl.h>
@@ -184,7 +183,7 @@ gp_port_library_list (GPPortInfoList *list)
 	gp_port_info_set_type (info, GP_PORT_PTPIP);
 	gp_port_info_set_name (info, "");
 	gp_port_info_set_path (info, "^ptpip:");
-	CHECK (gp_port_info_list_append (list, info));
+	gp_port_info_list_append (list, info); /* do not check return */
 
 #ifdef HAVE_MDNS_BONJOUR
 	ret = DNSServiceBrowse (
